"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ApexParser_1 = require("../ApexParser");
const ThrowingErrorListener_1 = require("../ThrowingErrorListener");
const SyntaxErrorCounter_1 = require("./SyntaxErrorCounter");
test('Boolean Literal', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.cls", "true");
    const context = parser.literal();
    expect(errorCounter.getNumErrors()).toEqual(0);
    expect(context).toBeInstanceOf(ApexParser_1.LiteralContext);
    expect(context.BooleanLiteral()).toBeTruthy();
    expect(context.BooleanLiteral().text).toBe("true");
});
test('Expression', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.cls", "a * 5");
    const context = parser.expression();
    expect(errorCounter.getNumErrors()).toEqual(0);
    expect(context).toBeInstanceOf(ApexParser_1.Arth1ExpressionContext);
    const arthExpression = context;
    expect(arthExpression.expression().length).toBe(2);
});
test('Compilation Unit', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.cls", "public class Hello {}");
    const context = parser.compilationUnit();
    expect(context).toBeInstanceOf(ApexParser_1.CompilationUnitContext);
    expect(errorCounter.getNumErrors()).toEqual(0);
});
test('Compilation Unit (case insensitive)', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.cls", "Public CLASS Hello {}");
    const context = parser.compilationUnit();
    expect(context).toBeInstanceOf(ApexParser_1.CompilationUnitContext);
    expect(errorCounter.getNumErrors()).toEqual(0);
});
test('Compilation Unit (bug test)', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.cls", `public class Hello {
        public testMethod void func() {
            System.runAs(u) {
            }
        }
    }`);
    const context = parser.compilationUnit();
    expect(context).toBeInstanceOf(ApexParser_1.CompilationUnitContext);
    expect(errorCounter.getNumErrors()).toEqual(0);
});
test('Compilation Unit (inline SOQL)', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.cls", `public class Hello {
        public void func() {
            List<Account> accounts = [Select Id from Accounts];
        }
    }`);
    const context = parser.compilationUnit();
    expect(context).toBeInstanceOf(ApexParser_1.CompilationUnitContext);
    expect(errorCounter.getNumErrors()).toEqual(0);
});
test('Compilation Unit (throwing errors)', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.cls", "public class Hello {");
    parser.removeErrorListeners();
    parser.addErrorListener(new ThrowingErrorListener_1.ThrowingErrorListener());
    try {
        parser.compilationUnit();
        expect(true).toBe(false);
    }
    catch (ex) {
        expect(ex).toBeInstanceOf(ThrowingErrorListener_1.SyntaxException);
    }
});
test('Trigger Unit', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.trigger", "trigger test on Account (before update, after update) {}");
    const context = parser.triggerUnit();
    expect(context).toBeInstanceOf(ApexParser_1.TriggerUnitContext);
    expect(errorCounter.getNumErrors()).toEqual(0);
});
test('SOQL Query', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.soql", "Select Id from Account");
    const context = parser.query();
    expect(context).toBeInstanceOf(ApexParser_1.QueryContext);
    expect(errorCounter.getNumErrors()).toEqual(0);
});
test('SOQL Query Using Field function', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.soql", "Select Fields(All) from Account");
    const context = parser.query();
    expect(context).toBeInstanceOf(ApexParser_1.QueryContext);
    expect(errorCounter.getNumErrors()).toEqual(0);
});
test('CurrencyLiteral', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.soql", "SELECT Id FROM Account WHERE Amount > USD100.01 AND Amount < USD200");
    const context = parser.query();
    expect(context).toBeInstanceOf(ApexParser_1.QueryContext);
    expect(errorCounter.getNumErrors()).toEqual(0);
});
test('IdentifiersThatCouldBeCurrencyLiterals', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.apex", "USD100.name = 'name';");
    const context = parser.statement();
    expect(context).toBeInstanceOf(ApexParser_1.StatementContext);
    expect(errorCounter.getNumErrors()).toEqual(0);
});
test('DateTimeLiteral', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.soql", "SELECT Name, (SELECT Id FROM Account WHERE createdDate > 2020-01-01T12:00:00Z) FROM Opportunity");
    const context = parser.query();
    expect(context).toBeInstanceOf(ApexParser_1.QueryContext);
    expect(errorCounter.getNumErrors()).toEqual(0);
});
test('testNegativeNumericLiteral', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.soql", "SELECT Name FROM Opportunity WHERE Value = -100.123");
    const context = parser.query();
    expect(context).toBeInstanceOf(ApexParser_1.QueryContext);
    expect(errorCounter.getNumErrors()).toEqual(0);
});
test('testLastQuarterKeyword', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.soql", "SELECT Id FROM Account WHERE DueDate = LAST_QUARTER");
    const context = parser.query();
    expect(context).toBeInstanceOf(ApexParser_1.QueryContext);
    expect(errorCounter.getNumErrors()).toEqual(0);
});
test('testSemiAllowedAsWhileBody', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.apex", "while (x++ < 10 && !(y-- < 0));");
    const context = parser.statement();
    expect(context).toBeInstanceOf(ApexParser_1.StatementContext);
    expect(errorCounter.getNumErrors()).toEqual(0);
});
test('testSemiAllowedAsForBody', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.apex", "for(x=0; x<10; x++);");
    const context = parser.statement();
    expect(context).toBeInstanceOf(ApexParser_1.StatementContext);
    expect(errorCounter.getNumErrors()).toEqual(0);
});
test('testSemiDisallowedAsGeneralStatement', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.apex", "if (x == 3); else { ; }");
    const context = parser.statement();
    expect(context).toBeInstanceOf(ApexParser_1.StatementContext);
    expect(errorCounter.getNumErrors()).toEqual(1);
});
test('testWhenLiteralParens', () => {
    const [parser, errorCounter] = (0, SyntaxErrorCounter_1.createParser)("test.apex", `
    switch on (x) {
        when 1 { return 1; }
        when ((2)) { return 2; }
        when (3), (4) { return 3; }
     }`);
    const context = parser.statement();
    expect(context).toBeInstanceOf(ApexParser_1.StatementContext);
    expect(errorCounter.getNumErrors()).toEqual(0);
});
//# sourceMappingURL=ApexParserTest.js.map